package kr.co.vividnext.sodalive.admin.member

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.member.MemberProvider
import kr.co.vividnext.sodalive.member.MemberRole
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.domain.Pageable
import org.springframework.data.repository.findByIdOrNull
import org.springframework.security.crypto.password.PasswordEncoder
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.time.ZoneId
import java.time.format.DateTimeFormatter

@Service
class AdminMemberService(
    private val repository: AdminMemberRepository,
    private val passwordEncoder: PasswordEncoder,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) {
    @Transactional
    fun updateMember(request: UpdateMemberRequest) {
        val member = repository.findByIdOrNull(request.id)
            ?: throw SodaException("해당 유저가 없습니다.")

        if (member.role != request.userType) {
            member.role = request.userType
        }
    }

    fun getMemberList(pageable: Pageable): GetAdminMemberListResponse {
        val totalCount = repository.getMemberTotalCount()
        val memberList = processMemberListToGetAdminMemberListResponseItemList(
            memberList = repository.getMemberList(
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )

        return GetAdminMemberListResponse(totalCount, memberList)
    }

    fun searchMember(searchWord: String, pageable: Pageable): GetAdminMemberListResponse {
        if (searchWord.length < 2) throw SodaException("2글자 이상 입력하세요.")
        val totalCount = repository.searchMemberTotalCount(searchWord = searchWord)
        val memberList = processMemberListToGetAdminMemberListResponseItemList(
            memberList = repository.searchMember(
                searchWord = searchWord,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong()
            )
        )

        return GetAdminMemberListResponse(totalCount, memberList)
    }

    fun getCreatorList(pageable: Pageable): GetAdminMemberListResponse {
        val totalCount = repository.getMemberTotalCount(role = MemberRole.CREATOR)
        val creatorList = processMemberListToGetAdminMemberListResponseItemList(
            memberList = repository.getMemberList(
                offset = pageable.offset,
                limit = pageable.pageSize.toLong(),
                role = MemberRole.CREATOR
            )
        )

        return GetAdminMemberListResponse(totalCount, creatorList)
    }

    fun searchCreator(searchWord: String, pageable: Pageable): GetAdminMemberListResponse {
        if (searchWord.length < 2) throw SodaException("2글자 이상 입력하세요.")
        val totalCount = repository.searchMemberTotalCount(searchWord = searchWord, role = MemberRole.CREATOR)
        val creatorList = processMemberListToGetAdminMemberListResponseItemList(
            memberList = repository.searchMember(
                searchWord = searchWord,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong(),
                role = MemberRole.CREATOR
            )
        )

        return GetAdminMemberListResponse(totalCount, creatorList)
    }

    private fun processMemberListToGetAdminMemberListResponseItemList(
        memberList: List<Member>
    ): List<GetAdminMemberListResponseItem> {
        return memberList
            .asSequence()
            .map {
                val userType = when (it.role) {
                    MemberRole.ADMIN -> "관리자"
                    MemberRole.USER -> "일반회원"
                    MemberRole.CREATOR -> "크리에이터"
                    MemberRole.AGENT -> "에이전트"
                    MemberRole.BOT -> "봇"
                }

                val loginType = when (it.provider) {
                    MemberProvider.EMAIL -> "이메일"
                    MemberProvider.KAKAO -> "카카오"
                    MemberProvider.GOOGLE -> "구글"
                    MemberProvider.APPLE -> "애플"
                }

                val signUpDate = it.createdAt!!
                    .atZone(ZoneId.of("UTC"))
                    .withZoneSameInstant(ZoneId.of("Asia/Seoul"))
                    .format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm"))

                val signOutDate = if (it.signOutReasons.isNotEmpty()) {
                    it.signOutReasons.last().createdAt!!
                        .atZone(ZoneId.of("UTC"))
                        .withZoneSameInstant(ZoneId.of("Asia/Seoul"))
                        .format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm"))
                } else {
                    ""
                }

                GetAdminMemberListResponseItem(
                    id = it.id!!,
                    email = it.email,
                    nickname = it.nickname,
                    profileUrl = if (it.profileImage != null) {
                        "$cloudFrontHost/${it.profileImage}"
                    } else {
                        "$cloudFrontHost/profile/default-profile.png"
                    },
                    userType = userType,
                    loginType = loginType,
                    container = it.container,
                    auth = it.auth != null,
                    signUpDate = signUpDate,
                    signOutDate = signOutDate,
                    isActive = it.isActive
                )
            }
            .toList()
    }

    fun getCreatorAllList(): List<GetAdminCreatorAllListResponse> {
        return repository.getCreatorAllList()
    }

    @Transactional
    fun resetPassword(request: ResetPasswordRequest) {
        val member = repository.findByIdAndActive(memberId = request.memberId)
            ?: throw SodaException("잘못된 회원정보입니다.\n다시 시도해 주세요.")

        member.password = passwordEncoder.encode(member.email.split("@")[0])
    }
}
