package kr.co.vividnext.sodalive.admin.statistics.ad

import com.querydsl.core.types.dsl.CaseBuilder
import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.NumberExpression
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.marketing.AdTrackingHistoryType
import kr.co.vividnext.sodalive.marketing.QAdTrackingHistory.adTrackingHistory
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
class AdminAdStatisticsRepository(private val queryFactory: JPAQueryFactory) {
    fun getAdStatisticsDataTotalCount(
        startDate: LocalDateTime,
        endDate: LocalDateTime
    ): Int {
        return queryFactory
            .select(adTrackingHistory.pid)
            .from(adTrackingHistory)
            .where(
                adTrackingHistory.createdAt.goe(startDate),
                adTrackingHistory.createdAt.loe(endDate)
            )
            .groupBy(
                getFormattedDate(adTrackingHistory.createdAt),
                adTrackingHistory.mediaGroup,
                adTrackingHistory.pid,
                adTrackingHistory.pidName
            )
            .fetch()
            .size
    }

    fun getAdStatisticsDataList(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        offset: Long,
        limit: Long
    ): List<GetAdminAdStatisticsItem> {
        val signUpCount = CaseBuilder()
            .`when`(adTrackingHistory.type.eq(AdTrackingHistoryType.SIGNUP))
            .then(1)
            .otherwise(0)
            .sum()

        val launchCount = CaseBuilder()
            .`when`(adTrackingHistory.type.eq(AdTrackingHistoryType.APP_LAUNCH))
            .then(1)
            .otherwise(0)
            .sum()

        val loginCount = CaseBuilder()
            .`when`(adTrackingHistory.type.eq(AdTrackingHistoryType.LOGIN))
            .then(1)
            .otherwise(0)
            .sum()

        val firstPaymentCount = CaseBuilder()
            .`when`(adTrackingHistory.type.eq(AdTrackingHistoryType.FIRST_PAYMENT))
            .then(1)
            .otherwise(0)
            .sum()

        val firstPaymentTotalAmount = CaseBuilder()
            .`when`(adTrackingHistory.type.eq(AdTrackingHistoryType.FIRST_PAYMENT))
            .then(adTrackingHistory.price)
            .otherwise(Expressions.constant(0.0))
            .sum()

        val repeatPaymentCount = CaseBuilder()
            .`when`(adTrackingHistory.type.eq(AdTrackingHistoryType.REPEAT_PAYMENT))
            .then(1)
            .otherwise(0)
            .sum()

        val repeatPaymentTotalAmount = CaseBuilder()
            .`when`(adTrackingHistory.type.eq(AdTrackingHistoryType.REPEAT_PAYMENT))
            .then(adTrackingHistory.price)
            .otherwise(Expressions.constant(0.0))
            .sum()

        val allPaymentCount = CaseBuilder()
            .`when`(
                adTrackingHistory.type.eq(AdTrackingHistoryType.FIRST_PAYMENT)
                    .or(adTrackingHistory.type.eq(AdTrackingHistoryType.REPEAT_PAYMENT))
            )
            .then(1)
            .otherwise(0)
            .sum()

        val allPaymentTotalAmount = CaseBuilder()
            .`when`(
                adTrackingHistory.type.eq(AdTrackingHistoryType.FIRST_PAYMENT)
                    .or(adTrackingHistory.type.eq(AdTrackingHistoryType.REPEAT_PAYMENT))
            )
            .then(adTrackingHistory.price)
            .otherwise(Expressions.constant(0.0))
            .sum()

        return queryFactory
            .select(
                QGetAdminAdStatisticsItem(
                    getFormattedDate(adTrackingHistory.createdAt),
                    adTrackingHistory.mediaGroup,
                    adTrackingHistory.pid,
                    adTrackingHistory.pidName,
                    launchCount,
                    loginCount,
                    signUpCount,
                    firstPaymentCount,
                    roundedValueDecimalPlaces2(firstPaymentTotalAmount),
                    repeatPaymentCount,
                    roundedValueDecimalPlaces2(repeatPaymentTotalAmount),
                    allPaymentCount,
                    roundedValueDecimalPlaces2(allPaymentTotalAmount)
                )
            )
            .from(adTrackingHistory)
            .where(
                adTrackingHistory.createdAt.goe(startDate),
                adTrackingHistory.createdAt.loe(endDate)
            )
            .groupBy(
                getFormattedDate(adTrackingHistory.createdAt),
                adTrackingHistory.mediaGroup,
                adTrackingHistory.pid,
                adTrackingHistory.pidName
            )
            .orderBy(getFormattedDate(adTrackingHistory.createdAt).desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }

    private fun getFormattedDate(dateTimePath: DateTimePath<LocalDateTime>): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTimePath,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d"
        )
    }

    private fun roundedValueDecimalPlaces2(valueExpression: NumberExpression<Double>): NumberExpression<Double> {
        return Expressions.numberTemplate(
            Double::class.java,
            "ROUND({0}, {1})",
            valueExpression,
            2
        )
    }
}
