package kr.co.vividnext.sodalive.agora

import org.springframework.stereotype.Component

@Component
class RtcTokenBuilder {
    /**
     * Builds an RTC token using an int uid.
     *
     * @param appId The App ID issued to you by Agora.
     * @param appCertificate Certificate of the application that you registered in
     * the Agora Dashboard.
     * @param channelName The unique channel name for the AgoraRTC session in the string format. The string length must be less than 64 bytes. Supported character scopes are:
     *
     *  *  The 26 lowercase English letters: a to z.
     *  *  The 26 uppercase English letters: A to Z.
     *  *  The 10 digits: 0 to 9.
     *  *  The space.
     *  *  "!", "#", "$", "%", "&", "(", ")", "+", "-", ":", ";", "<", "=", ".", ">", "?", "@", "[", "]", "^", "_", " {", "}", "|", "~", ",".
     *
     * @param uid  User ID. A 32-bit unsigned integer with a value ranging from
     * 1 to (2^32-1).
     * @param role The user role.
     *
     *  *  Role_Publisher = 1: RECOMMENDED. Use this role for a voice/video call or a live broadcast.
     *  *  Role_Subscriber = 2: ONLY use this role if your live-broadcast scenario requires authentication for [Hosting-in](https://docs.agora.io/en/Agora%20Platform/terms?platform=All%20Platforms#hosting-in). In order for this role to take effect, please contact our support team to enable authentication for Hosting-in for you. Otherwise, Role_Subscriber still has the same privileges as Role_Publisher.
     *
     * @param privilegeTs Represented by the number of seconds elapsed since 1/1/1970.
     * If, for example, you want to access the Agora Service within 10 minutes
     * after the token is generated, set expireTimestamp as the current time stamp
     * + 600 (seconds).
     */
    fun buildTokenWithUid(
        appId: String,
        appCertificate: String,
        channelName: String,
        uid: Int,
        privilegeTs: Int
    ): String {
        val account = if (uid == 0) "" else uid.toString()
        return buildTokenWithUserAccount(
            appId,
            appCertificate,
            channelName,
            account,
            privilegeTs
        )
    }

    /**
     * Builds an RTC token using a string userAccount.
     *
     * @param appId The App ID issued to you by Agora.
     * @param appCertificate Certificate of the application that you registered in
     * the Agora Dashboard.
     * @param channelName The unique channel name for the AgoraRTC session in the string format. The string length must be less than 64 bytes. Supported character scopes are:
     *
     *  *  The 26 lowercase English letters: a to z.
     *  *  The 26 uppercase English letters: A to Z.
     *  *  The 10 digits: 0 to 9.
     *  *  The space.
     *  *  "!", "#", "$", "%", "&", "(", ")", "+", "-", ":", ";", "<", "=", ".", ">", "?", "@", "[", "]", "^", "_", " {", "}", "|", "~", ",".
     *
     * @param account  The user account.
     * @param role The user role.
     *
     *  *  Role_Publisher = 1: RECOMMENDED. Use this role for a voice/video call or a live broadcast.
     *  *  Role_Subscriber = 2: ONLY use this role if your live-broadcast scenario requires authentication for [Hosting-in](https://docs.agora.io/en/Agora%20Platform/terms?platform=All%20Platforms#hosting-in). In order for this role to take effect, please contact our support team to enable authentication for Hosting-in for you. Otherwise, Role_Subscriber still has the same privileges as Role_Publisher.
     *
     * @param privilegeTs represented by the number of seconds elapsed since 1/1/1970.
     * If, for example, you want to access the Agora Service within 10 minutes
     * after the token is generated, set expireTimestamp as the current time stamp
     * + 600 (seconds).
     */
    fun buildTokenWithUserAccount(
        appId: String,
        appCertificate: String,
        channelName: String,
        account: String,
        privilegeTs: Int
    ): String {
        // Assign appropriate access privileges to each role.
        val builder = AccessToken(appId, appCertificate, channelName, account)
        builder.addPrivilege(AccessToken.Privileges.JoinChannel, privilegeTs)
        builder.addPrivilege(AccessToken.Privileges.PublishAudioStream, privilegeTs)
        builder.addPrivilege(AccessToken.Privileges.PublishVideoStream, privilegeTs)
        builder.addPrivilege(AccessToken.Privileges.PublishDataStream, privilegeTs)

        return try {
            builder.build()
        } catch (e: Exception) {
            e.printStackTrace()
            ""
        }
    }
}
