package kr.co.vividnext.sodalive.creator.admin.signature

import com.amazonaws.services.s3.model.ObjectMetadata
import kr.co.vividnext.sodalive.aws.s3.S3Uploader
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.live.signature.SignatureCan
import kr.co.vividnext.sodalive.live.signature.SignatureCanSortType
import kr.co.vividnext.sodalive.member.MemberRepository
import kr.co.vividnext.sodalive.utils.generateFileName
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.domain.Pageable
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import org.springframework.web.multipart.MultipartFile

@Service
class CreatorAdminSignatureService(
    private val repository: CreatorAdminSignatureRepository,
    private val memberRepository: MemberRepository,

    private val s3Uploader: S3Uploader,

    @Value("\${cloud.aws.s3.bucket}")
    private val bucket: String,
    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) {
    fun getSignatureList(
        pageable: Pageable,
        sortType: SignatureCanSortType,
        memberId: Long
    ): GetSignatureListResponse {
        val totalCount = repository.getSignatureListTotalCount(memberId = memberId)
        val items = repository.getSignatureList(
            memberId = memberId,
            sortType = sortType,
            imageHost = imageHost,
            offset = pageable.offset,
            limit = pageable.pageSize.toLong()
        )

        return GetSignatureListResponse(totalCount, items)
    }

    @Transactional
    fun createSignature(can: Int, time: Int, image: MultipartFile, memberId: Long, isAdult: Boolean) {
        if (can <= 0) throw SodaException("1캔 이상 설정할 수 있습니다.")
        if (time < 3 || time > 20) throw SodaException("시간은 3초 이상 20초 이하로 설정할 수 있습니다.")

        val member = memberRepository.findCreatorByIdOrNull(memberId = memberId)
            ?: throw SodaException("잘못된 접근입니다.")

        val signatureCan = SignatureCan(can = can, time = time, isAdult = isAdult)
        signatureCan.creator = member
        repository.save(signatureCan)

        val metadata = ObjectMetadata()
        metadata.contentLength = image.size
        val imagePath = s3Uploader.upload(
            inputStream = image.inputStream,
            bucket = bucket,
            filePath = "signature_can/${signatureCan.id}/${generateFileName()}",
            metadata = metadata
        )

        signatureCan.image = imagePath
    }

    @Transactional
    fun modifySignature(
        id: Long,
        can: Int?,
        time: Int?,
        image: MultipartFile?,
        isActive: Boolean?,
        memberId: Long,
        isAdult: Boolean?
    ) {
        val signatureCan = repository.findSignatureByIdOrNull(id = id, memberId = memberId)
            ?: throw SodaException("잘못된 요청입니다.")

        if (can != null) {
            if (can <= 0) throw SodaException("1캔 이상 설정할 수 있습니다.")
            signatureCan.can = can
        }

        if (time != null) {
            if (time < 3 || time > 20) throw SodaException("시간은 3초 이상 20초 이하로 설정할 수 있습니다.")
            signatureCan.time = time
        }

        if (isActive != null) {
            signatureCan.isActive = isActive
        }

        if (isAdult != null) {
            signatureCan.isAdult = isAdult
        }

        if (image != null) {
            val metadata = ObjectMetadata()
            metadata.contentLength = image.size
            val imagePath = s3Uploader.upload(
                inputStream = image.inputStream,
                bucket = bucket,
                filePath = "signature_can/${signatureCan.id}/${generateFileName()}",
                metadata = metadata
            )

            signatureCan.image = imagePath
        }
    }
}
