package kr.co.vividnext.sodalive.live.room.menu

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.live.roulette.RedisIdGenerator
import org.springframework.stereotype.Service

@Service
class LiveRoomMenuService(
    private val idGenerator: RedisIdGenerator,
    private val repository: LiveRoomMenuRepository
) {
    fun getAllLiveMenu(creatorId: Long, memberId: Long): List<GetMenuPresetResponse> {
        if (creatorId != memberId) throw SodaException("잘못된 요청입니다.")

        return repository.findByCreatorId(creatorId)
            .sortedBy { it.id }
            .asSequence()
            .map { GetMenuPresetResponse(id = it.id, menu = it.menu, isActive = it.isActive) }
            .toList()
    }

    fun createLiveMenu(memberId: Long, request: CreateLiveMenuRequest): Boolean {
        liveMenuValidate(menu = request.menu)

        val menuList = repository.findByCreatorId(creatorId = memberId)

        if (menuList.size >= 3) {
            throw SodaException("메뉴판의 최대개수는 3개입니다.")
        }

        if (request.isActive) {
            menuList.forEach {
                it.isActive = false
                repository.save(it)
            }
        }

        val menu = LiveRoomMenu(
            id = idGenerator.generateId(SEQUENCE_NAME),
            creatorId = memberId,
            isActive = request.isActive,
            menu = request.menu
        )

        repository.save(menu)
        return request.isActive
    }

    fun updateLiveMenu(memberId: Long, request: UpdateLiveMenuRequest) {
        liveMenuValidate(menu = request.menu)

        val menuList = repository.findByCreatorId(creatorId = memberId)
        if (menuList.isEmpty()) {
            throw SodaException("잘못된 요청입니다.")
        }

        menuList.forEach {
            if (it.id == request.id) {
                it.menu = request.menu
                it.isActive = request.isActive
                repository.save(it)
            } else if (request.isActive) {
                it.isActive = false
                repository.save(it)
            }
        }
    }

    fun getLiveMenu(creatorId: Long): GetMenuPresetResponse? {
        val menuList = repository.findByCreatorId(creatorId = creatorId)

        var activeMenu: LiveRoomMenu? = null
        for (menu in menuList) {
            if (menu.isActive) {
                activeMenu = menu
                break
            }
        }

        if (activeMenu == null || activeMenu.menu.isEmpty()) {
            return null
        }

        return GetMenuPresetResponse(id = activeMenu.id, menu = activeMenu.menu, isActive = activeMenu.isActive)
    }

    fun deactivateAll(memberId: Long) {
        val menuList = repository.findByCreatorId(creatorId = memberId)

        menuList.forEach {
            it.isActive = false
            repository.save(it)
        }
    }

    private fun liveMenuValidate(menu: String) {
        if (menu.isBlank()) {
            throw SodaException("메뉴판은 빈칸일 수 없습니다.")
        }
    }

    companion object {
        const val SEQUENCE_NAME = "LiveRoomMenu:sequence"
    }
}
