package kr.co.vividnext.sodalive.member

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.fcm.GetMessageRecipientPushTokenResponse
import kr.co.vividnext.sodalive.fcm.QGetMessageRecipientPushTokenResponse
import kr.co.vividnext.sodalive.fcm.QPushToken.pushToken
import kr.co.vividnext.sodalive.live.reservation.QLiveReservation.liveReservation
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.member.QMember.member
import kr.co.vividnext.sodalive.member.auth.QAuth.auth
import kr.co.vividnext.sodalive.member.block.BlockMemberRepository
import kr.co.vividnext.sodalive.member.block.QBlockMember.blockMember
import kr.co.vividnext.sodalive.member.following.QCreatorFollowing.creatorFollowing
import kr.co.vividnext.sodalive.member.nickname.QNicknameChangeLog.nicknameChangeLog
import kr.co.vividnext.sodalive.member.notification.QMemberNotification.memberNotification
import kr.co.vividnext.sodalive.message.QMessage.message
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository

@Repository
interface MemberRepository : JpaRepository<Member, Long>, MemberQueryRepository {
    fun findByEmail(email: String): Member?
    fun findByNickname(nickname: String): Member?
    fun findByGoogleId(googleId: String): Member?
    fun findByKakaoId(kakaoId: Long): Member?
}

interface MemberQueryRepository {
    fun findByNicknameAndOtherCondition(nickname: String, member: Member): List<Member>
    fun findCreatorByIdOrNull(memberId: Long): Member?
    fun getAllRecipientPushTokens(isAuth: Boolean?, container: String): List<List<String>>
    fun getCreateLiveRoomNotificationRecipientPushTokens(
        creatorId: Long,
        isAuth: Boolean,
        isAvailableJoinCreator: Boolean,
        container: String
    ): List<List<String>>

    fun getStartLiveRoomNotificationRecipientPushTokens(
        creatorId: Long,
        roomId: Long,
        isAuth: Boolean,
        isAvailableJoinCreator: Boolean,
        container: String
    ): List<List<String>>

    fun getUploadContentNotificationRecipientPushTokens(
        creatorId: Long,
        isAuth: Boolean,
        container: String
    ): List<List<String>>

    fun getMessageRecipientPushToken(messageId: Long): GetMessageRecipientPushTokenResponse?
    fun getIndividualRecipientPushTokens(recipients: List<Long>, isAuth: Boolean?): Map<String, List<List<String>>>
    fun getChangeNicknamePrice(memberId: Long): GetChangeNicknamePriceResponse
    fun getMemberByEmail(email: String): Member?

    fun getChangeNoticeRecipientPushTokens(creatorId: Long): Map<String, List<List<String>>>
    fun getPushTokenFromReservationList(roomId: Long): Map<String, List<List<String>>>
    fun getAuditionNoticeRecipientPushTokens(isAuth: Boolean): Map<String, List<List<String>>>

    fun getMemberProfile(memberId: Long, myMemberId: Long): GetMemberProfileResponse

    fun existsByNickname(nickname: String): Boolean
    fun findNicknamesWithPrefix(prefix: String): List<String>
}

@Repository
class MemberQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,
    private val blockMemberRepository: BlockMemberRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) : MemberQueryRepository {
    override fun findByNicknameAndOtherCondition(nickname: String, member: Member): List<Member> {
        var where = QMember.member.nickname.containsIgnoreCase(nickname)
            .and(QMember.member.id.ne(member.id))
            .and(QMember.member.role.ne(MemberRole.ADMIN))
            .and(QMember.member.role.ne(MemberRole.AGENT))
            .and(QMember.member.role.ne(MemberRole.BOT))

        if (member.role == MemberRole.USER) {
            where = where.and(QMember.member.role.ne(MemberRole.USER))
        }

        return queryFactory
            .selectFrom(QMember.member)
            .where(where)
            .fetch()
    }

    override fun findCreatorByIdOrNull(memberId: Long): Member? {
        return queryFactory
            .selectFrom(member)
            .where(
                member.id.eq(memberId)
                    .and(member.role.eq(MemberRole.CREATOR))
            )
            .fetchFirst()
    }

    override fun getAllRecipientPushTokens(isAuth: Boolean?, container: String): List<List<String>> {
        var where = member.isActive.isTrue
            .and(member.email.notIn("admin@sodalive.net"))
            .and(pushToken.deviceType.eq(container))

        if (isAuth != null) {
            where = if (isAuth) {
                where.and(auth.isNotNull)
            } else {
                where.and(auth.isNull)
            }
        }

        return queryFactory
            .select(pushToken.token)
            .from(member)
            .innerJoin(pushToken).on(member.id.eq(pushToken.member.id))
            .leftJoin(member.auth, auth)
            .where(where)
            .fetch()
            .toSet()
            .chunked(500)
    }

    override fun getCreateLiveRoomNotificationRecipientPushTokens(
        creatorId: Long,
        isAuth: Boolean,
        isAvailableJoinCreator: Boolean,
        container: String
    ): List<List<String>> {
        val member = QMember.member
        val creator = QMember.member

        var where = creatorFollowing.isActive.isTrue
            .and(creatorFollowing.member.isActive.isTrue)
            .and(creatorFollowing.creator.id.eq(creatorId))
            .and(creatorFollowing.member.notification.live.isTrue)
            .and(creatorFollowing.member.email.notIn("admin@sodalive.net"))
            .and(
                creatorFollowing.member.id.notIn(
                    blockMemberRepository.getBlockedMemberIdList(creatorId)
                )
            )
            .and(creatorFollowing.isNotify.isTrue)
            .and(pushToken.deviceType.eq(container))
            .or(member.id.eq(4))

        if (isAuth) {
            where = where.and(auth.isNotNull)
        }

        if (!isAvailableJoinCreator) {
            where = where.and(creatorFollowing.member.role.ne(MemberRole.CREATOR))
        }

        return queryFactory
            .select(pushToken.token)
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.creator, creator)
            .innerJoin(creatorFollowing.member, member)
            .innerJoin(creatorFollowing.member.notification, memberNotification)
            .innerJoin(pushToken).on(creatorFollowing.member.id.eq(pushToken.member.id))
            .leftJoin(creatorFollowing.member.auth, auth)
            .where(where)
            .fetch()
            .toSet()
            .chunked(500)
    }

    override fun getStartLiveRoomNotificationRecipientPushTokens(
        creatorId: Long,
        roomId: Long,
        isAuth: Boolean,
        isAvailableJoinCreator: Boolean,
        container: String
    ): List<List<String>> {
        val member = QMember.member
        val creator = QMember.member

        var where = creatorFollowing.isActive.isTrue
            .and(creatorFollowing.member.isActive.isTrue)
            .and(creatorFollowing.creator.id.eq(creatorId))
            .and(creatorFollowing.member.notification.live.isTrue)
            .and(creatorFollowing.member.email.notIn("admin@sodalive.net"))
            .and(
                creatorFollowing.member.id.notIn(
                    blockMemberRepository.getBlockedMemberIdList(creatorId)
                )
            )
            .and(creatorFollowing.isNotify.isTrue)
            .and(pushToken.deviceType.eq(container))
            .or(creatorFollowing.member.id.eq(4))

        if (isAuth) {
            where = where.and(auth.isNotNull)
        }

        if (!isAvailableJoinCreator) {
            where = where.and(creatorFollowing.member.role.ne(MemberRole.CREATOR))
        }

        val followingMemberPushToken = queryFactory
            .select(pushToken.token)
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.creator, creator)
            .innerJoin(creatorFollowing.member, member)
            .innerJoin(creatorFollowing.member.notification, memberNotification)
            .innerJoin(pushToken).on(creatorFollowing.member.id.eq(pushToken.member.id))
            .leftJoin(creatorFollowing.member.auth, auth)
            .where(where)
            .fetch()

        where = liveReservation.isActive.isTrue
            .and(liveReservation.member.notification.live.isTrue)
            .and(liveReservation.member.email.notIn("admin@sodalive.net"))
            .and(liveReservation.room.id.eq(roomId))
            .and(
                liveReservation.member.id.notIn(
                    blockMemberRepository.getBlockedMemberIdList(creatorId)
                )
            )
            .and(pushToken.deviceType.eq(container))

        if (isAuth) {
            where = where.and(auth.isNotNull)
        }

        val reservationMemberPushToken = queryFactory
            .select(pushToken.token)
            .from(liveReservation)
            .innerJoin(liveReservation.member, member)
            .innerJoin(liveReservation.member.notification, memberNotification)
            .innerJoin(pushToken).on(liveReservation.member.id.eq(pushToken.member.id))
            .leftJoin(liveReservation.member.auth, auth)
            .where(where)
            .fetch()

        return (followingMemberPushToken + reservationMemberPushToken)
            .toSet()
            .chunked(500)
    }

    override fun getUploadContentNotificationRecipientPushTokens(
        creatorId: Long,
        isAuth: Boolean,
        container: String
    ): List<List<String>> {
        val member = QMember.member
        val creator = QMember.member

        var where = creatorFollowing.isActive.isTrue
            .and(creatorFollowing.creator.id.eq(creatorId))
            .and(creatorFollowing.member.email.notIn("admin@sodalive.net"))
            .and(creatorFollowing.member.notification.uploadContent.isTrue)
            .and(
                creatorFollowing.member.id.notIn(
                    blockMemberRepository.getBlockedMemberIdList(creatorId)
                )
            )
            .and(creatorFollowing.isNotify.isTrue)
            .and(pushToken.deviceType.eq(container))
            .or(member.id.eq(4))

        if (isAuth) {
            where = where.and(auth.isNotNull)
        }

        return queryFactory
            .select(pushToken.token)
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.creator, creator)
            .innerJoin(creatorFollowing.member, member)
            .innerJoin(member.notification, memberNotification)
            .innerJoin(pushToken).on(creatorFollowing.member.id.eq(pushToken.member.id))
            .leftJoin(member.auth, auth)
            .where(where)
            .fetch()
            .toSet()
            .chunked(500)
    }

    override fun getMessageRecipientPushToken(messageId: Long): GetMessageRecipientPushTokenResponse? {
        return queryFactory
            .select(
                QGetMessageRecipientPushTokenResponse(
                    pushToken.token,
                    pushToken.deviceType
                )
            )
            .from(message)
            .innerJoin(message.recipient, member)
            .innerJoin(member.notification, memberNotification)
            .innerJoin(pushToken).on(member.id.eq(pushToken.member.id))
            .where(
                message.id.eq(messageId)
                    .and(memberNotification.message.isTrue)
            )
            .fetchFirst()
    }

    override fun getIndividualRecipientPushTokens(
        recipients: List<Long>,
        isAuth: Boolean?
    ): Map<String, List<List<String>>> {
        var where = member.isActive.isTrue
            .and(member.email.notIn("admin@sodalive.net"))
            .and(member.id.`in`(*recipients.toTypedArray()))

        if (isAuth != null) {
            where = if (isAuth) {
                where.and(auth.isNotNull)
            } else {
                where.and(auth.isNull)
            }
        }

        val aosPushTokens = queryFactory
            .select(pushToken.token)
            .from(member)
            .innerJoin(pushToken).on(member.id.eq(pushToken.member.id))
            .leftJoin(member.auth, auth)
            .where(where.and(pushToken.deviceType.eq("aos")))
            .fetch()
            .toSet()
            .chunked(500)

        val iosPushTokens = queryFactory
            .select(pushToken.token)
            .from(member)
            .innerJoin(pushToken).on(member.id.eq(pushToken.member.id))
            .leftJoin(member.auth, auth)
            .where(where.and(pushToken.deviceType.eq("ios")))
            .fetch()
            .toSet()
            .chunked(500)

        return mapOf("aos" to aosPushTokens, "ios" to iosPushTokens)
    }

    override fun getChangeNicknamePrice(memberId: Long): GetChangeNicknamePriceResponse {
        val changeCount = queryFactory
            .select(nicknameChangeLog.id)
            .from(nicknameChangeLog)
            .where(nicknameChangeLog.member.id.eq(memberId))
            .fetch()
            .count()

        return GetChangeNicknamePriceResponse(
            price = if (changeCount > 0) {
                100
            } else {
                0
            }
        )
    }

    override fun getMemberByEmail(email: String): Member? {
        return queryFactory
            .selectFrom(member)
            .where(member.email.eq(email))
            .fetchOne()
    }

    override fun getChangeNoticeRecipientPushTokens(creatorId: Long): Map<String, List<List<String>>> {
        val member = QMember.member
        val creator = QMember.member

        val where = creatorFollowing.isActive.isTrue
            .and(creatorFollowing.member.isActive.isTrue)
            .and(creatorFollowing.creator.id.eq(creatorId))
            .and(creatorFollowing.member.email.notIn("admin@sodalive.net"))
            .and(
                creatorFollowing.member.id.notIn(
                    blockMemberRepository.getBlockedMemberIdList(creatorId)
                )
            )
            .and(creatorFollowing.isNotify.isTrue)

        val aosPushTokens = queryFactory
            .select(pushToken.token)
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.creator, creator)
            .innerJoin(creatorFollowing.member, member)
            .innerJoin(pushToken).on(creatorFollowing.member.id.eq(pushToken.member.id))
            .where(where.and(pushToken.deviceType.eq("aos")))
            .fetch()
            .toSet()
            .chunked(500)

        val iosPushTokens = queryFactory
            .select(pushToken.token)
            .from(creatorFollowing)
            .innerJoin(creatorFollowing.creator, creator)
            .innerJoin(creatorFollowing.member, member)
            .innerJoin(pushToken).on(creatorFollowing.member.id.eq(pushToken.member.id))
            .where(where.and(pushToken.deviceType.eq("ios")))
            .fetch()
            .toSet()
            .chunked(500)

        return mapOf("aos" to aosPushTokens, "ios" to iosPushTokens)
    }

    override fun getPushTokenFromReservationList(roomId: Long): Map<String, List<List<String>>> {
        val where = liveRoom.id.eq(roomId)
            .and(liveReservation.isActive.isTrue)

        val aosPushTokens = queryFactory
            .select(pushToken.token)
            .from(liveReservation)
            .innerJoin(liveReservation.room, liveRoom)
            .innerJoin(liveReservation.member, member)
            .innerJoin(pushToken).on(member.id.eq(pushToken.member.id))
            .where(where.and(pushToken.deviceType.eq("aos")))
            .fetch()
            .toSet()
            .chunked(500)

        val iosPushTokens = queryFactory
            .select(pushToken.token)
            .from(liveReservation)
            .innerJoin(liveReservation.room, liveRoom)
            .innerJoin(liveReservation.member, member)
            .innerJoin(pushToken).on(member.id.eq(pushToken.member.id))
            .where(where.and(pushToken.deviceType.eq("ios")))
            .fetch()
            .toSet()
            .chunked(500)

        return mapOf("aos" to aosPushTokens, "ios" to iosPushTokens)
    }

    override fun getAuditionNoticeRecipientPushTokens(isAuth: Boolean): Map<String, List<List<String>>> {
        var where = memberNotification.audition.isTrue

        if (isAuth) {
            where = where.and(auth.isNotNull)
        }

        val aosPushTokens = queryFactory
            .select(pushToken.token)
            .from(member)
            .innerJoin(pushToken).on(member.id.eq(pushToken.member.id))
            .leftJoin(member.auth, auth)
            .leftJoin(memberNotification).on(memberNotification.member.id.eq(member.id))
            .where(where.and(pushToken.deviceType.eq("aos")))
            .fetch()
            .toSet()
            .chunked(500)

        val iosPushTokens = queryFactory
            .select(pushToken.token)
            .from(member)
            .innerJoin(pushToken).on(member.id.eq(pushToken.member.id))
            .leftJoin(member.auth, auth)
            .leftJoin(memberNotification).on(memberNotification.member.id.eq(member.id))
            .where(where.and(pushToken.deviceType.eq("ios")))
            .fetch()
            .toSet()
            .chunked(500)

        return mapOf("aos" to aosPushTokens, "ios" to iosPushTokens)
    }

    override fun getMemberProfile(memberId: Long, myMemberId: Long): GetMemberProfileResponse {
        return queryFactory
            .select(
                QGetMemberProfileResponse(
                    member.id,
                    member.nickname,
                    member.profileImage.prepend("/").prepend(cloudFrontHost),
                    blockMember.id.isNotNull
                )
            )
            .from(member)
            .leftJoin(blockMember)
            .on(
                member.id.eq(blockMember.blockedMember.id)
                    .and(blockMember.isActive.isTrue)
                    .and(blockMember.member.id.eq(myMemberId))
            )
            .where(member.id.eq(memberId))
            .fetchFirst()
    }

    override fun existsByNickname(nickname: String): Boolean {
        return queryFactory
            .selectOne()
            .from(member)
            .where(
                member.nickname.eq(nickname),
                member.isActive.isTrue
            )
            .fetchFirst() != null
    }

    override fun findNicknamesWithPrefix(prefix: String): List<String> {
        return queryFactory
            .select(member.nickname)
            .from(member)
            .where(
                member.nickname.startsWith(prefix),
                member.isActive.isTrue
            )
            .fetch()
    }
}
