package kr.co.vividnext.sodalive.useraction

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kr.co.vividnext.sodalive.fcm.FcmService
import kr.co.vividnext.sodalive.point.MemberPoint
import kr.co.vividnext.sodalive.point.MemberPointRepository
import kr.co.vividnext.sodalive.point.PointGrantLog
import kr.co.vividnext.sodalive.point.PointGrantLogRepository
import kr.co.vividnext.sodalive.point.PointRewardPolicyRepository
import org.springframework.stereotype.Service
import java.time.LocalDateTime

@Service
class UserActionService(
    private val repository: UserActionLogRepository,
    private val policyRepository: PointRewardPolicyRepository,
    private val grantLogRepository: PointGrantLogRepository,
    private val memberPointRepository: MemberPointRepository,

    private val fcmService: FcmService
) {

    private val coroutineScope = CoroutineScope(Dispatchers.IO)

    fun recordAction(memberId: Long, actionType: ActionType, orderId: Long? = null, pushToken: String? = null) {
        coroutineScope.launch {
            val now = LocalDateTime.now()
            repository.save(UserActionLog(memberId, actionType))

            val policy = policyRepository.findByActionTypeAndIsActiveTrue(actionType, now)
            if (policy != null) {
                val policyType = policy.policyType
                val todayAt15 = now.toLocalDate().atTime(15, 0)
                val policyTypeDailyStartDate = if (now.toLocalTime().isBefore(todayAt15.toLocalTime())) {
                    now.toLocalDate().minusDays(1).atTime(15, 0)
                } else {
                    todayAt15
                }

                val actionCount = repository.countByMemberIdAndActionTypeAndCreatedAtBetween(
                    memberId = memberId,
                    actionType = actionType,
                    startDate = if (policyType == PolicyType.DAILY) policyTypeDailyStartDate else policy.startDate,
                    endDate = policy.endDate ?: now
                )
                if (actionCount < policy.threshold) return@launch

                val grantedCount = grantLogRepository.countByMemberIdAndPolicyIdAndStartDate(
                    memberId,
                    policy.id!!,
                    startDate = if (policyType == PolicyType.DAILY) policyTypeDailyStartDate else policy.startDate
                )
                if (grantedCount >= policy.availableCount) return@launch

                memberPointRepository.save(
                    MemberPoint(
                        memberId = memberId,
                        point = policy.pointAmount,
                        actionType = actionType,
                        expiresAt = now.plusDays(3)
                    )
                )

                grantLogRepository.save(
                    PointGrantLog(
                        memberId = memberId,
                        point = policy.pointAmount,
                        actionType = actionType,
                        policyId = policy.id!!,
                        orderId = orderId
                    )
                )

                if (pushToken != null) {
                    fcmService.sendPointGranted(pushToken, policy.pointAmount)
                }
            }
        }
    }
}
